# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass
from typing import Any, Dict
import zaber_bson


@dataclass
class UnitGetSymbolResponse:

    symbol: str = ""

    @staticmethod
    def zero_values() -> 'UnitGetSymbolResponse':
        return UnitGetSymbolResponse(
            symbol="",
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'UnitGetSymbolResponse':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return UnitGetSymbolResponse.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'symbol': str(self.symbol or ''),
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'UnitGetSymbolResponse':
        return UnitGetSymbolResponse(
            symbol=data.get('symbol'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.symbol is not None:
            if not isinstance(self.symbol, str):
                raise ValueError(f'Property "Symbol" of "UnitGetSymbolResponse" is not a string.')



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

from typing import List, Optional
from .call import call, call_async
from .dto import requests as dto
from .dto.requests import TestVariant


class VariantTestApi:
    """
    Test case for certain library features. Not public API; do not use.
    """

    @staticmethod
    def mutate_variant(
            the_variant: TestVariant
    ) -> Optional[TestVariant]:
        """
        Tests serialization of variants.

        Args:
            the_variant: Test value to modify and return.

        Returns:
            Mutated input value.
        """
        request = dto.TestDtoWithVariants(
            the_variant=the_variant,
        )
        response = call(
            "test/mutate_variant",
            request,
            dto.TestDtoWithVariants.from_binary)
        return response.the_variant

    @staticmethod
    async def mutate_variant_async(
            the_variant: TestVariant
    ) -> Optional[TestVariant]:
        """
        Tests serialization of variants.

        Args:
            the_variant: Test value to modify and return.

        Returns:
            Mutated input value.
        """
        request = dto.TestDtoWithVariants(
            the_variant=the_variant,
        )
        response = await call_async(
            "test/mutate_variant",
            request,
            dto.TestDtoWithVariants.from_binary)
        return response.the_variant

    @staticmethod
    def mutate_variant_array(
            variant_array: List[TestVariant]
    ) -> List[TestVariant]:
        """
        Tests serialization of arrays of variants.

        Args:
            variant_array: Test values to modify and return.

        Returns:
            Mutated input values.
        """
        request = dto.TestDtoWithVariants(
            variant_array=variant_array,
        )
        response = call(
            "test/mutate_variant",
            request,
            dto.TestDtoWithVariants.from_binary)
        return response.variant_array

    @staticmethod
    async def mutate_variant_array_async(
            variant_array: List[TestVariant]
    ) -> List[TestVariant]:
        """
        Tests serialization of arrays of variants.

        Args:
            variant_array: Test values to modify and return.

        Returns:
            Mutated input values.
        """
        request = dto.TestDtoWithVariants(
            variant_array=variant_array,
        )
        response = await call_async(
            "test/mutate_variant",
            request,
            dto.TestDtoWithVariants.from_binary)
        return response.variant_array



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional
import decimal
from collections.abc import Iterable
import zaber_bson
from ..ascii.measurement_sequence import MeasurementSequence
from ..measurement import Measurement


@dataclass
class PvtGenerateVelocitiesAndTimesRequest:

    positions: List[MeasurementSequence] = field(default_factory=list)

    target_speed: Measurement = field(default_factory=Measurement.zero_values)

    target_acceleration: Measurement = field(default_factory=Measurement.zero_values)

    resample_number: Optional[int] = None

    @staticmethod
    def zero_values() -> 'PvtGenerateVelocitiesAndTimesRequest':
        return PvtGenerateVelocitiesAndTimesRequest(
            positions=[],
            target_speed=Measurement.zero_values(),
            target_acceleration=Measurement.zero_values(),
            resample_number=None,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'PvtGenerateVelocitiesAndTimesRequest':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return PvtGenerateVelocitiesAndTimesRequest.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'positions': [item.to_dict() for item in self.positions] if self.positions is not None else [],
            'targetSpeed': self.target_speed.to_dict(),
            'targetAcceleration': self.target_acceleration.to_dict(),
            'resampleNumber': int(self.resample_number) if self.resample_number is not None else None,
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'PvtGenerateVelocitiesAndTimesRequest':
        return PvtGenerateVelocitiesAndTimesRequest(
            positions=[MeasurementSequence.from_dict(item) for item in data.get('positions')],  # type: ignore
            target_speed=Measurement.from_dict(data.get('targetSpeed')),  # type: ignore
            target_acceleration=Measurement.from_dict(data.get('targetAcceleration')),  # type: ignore
            resample_number=data.get('resampleNumber'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.positions is not None:
            if not isinstance(self.positions, Iterable):
                raise ValueError('Property "Positions" of "PvtGenerateVelocitiesAndTimesRequest" is not iterable.')

            for i, positions_item in enumerate(self.positions):
                if positions_item is None:
                    raise ValueError(f'Item {i} in property "Positions" of "PvtGenerateVelocitiesAndTimesRequest" is None.')

                if not isinstance(positions_item, MeasurementSequence):
                    raise ValueError(f'Item {i} in property "Positions" of "PvtGenerateVelocitiesAndTimesRequest" is not an instance of "MeasurementSequence".')

                positions_item.validate()

        if self.target_speed is None:
            raise ValueError(f'Property "TargetSpeed" of "PvtGenerateVelocitiesAndTimesRequest" is None.')

        if not isinstance(self.target_speed, Measurement):
            raise ValueError(f'Property "TargetSpeed" of "PvtGenerateVelocitiesAndTimesRequest" is not an instance of "Measurement".')

        self.target_speed.validate()

        if self.target_acceleration is None:
            raise ValueError(f'Property "TargetAcceleration" of "PvtGenerateVelocitiesAndTimesRequest" is None.')

        if not isinstance(self.target_acceleration, Measurement):
            raise ValueError(f'Property "TargetAcceleration" of "PvtGenerateVelocitiesAndTimesRequest" is not an instance of "Measurement".')

        self.target_acceleration.validate()

        if self.resample_number is not None:
            if not isinstance(self.resample_number, (int, float, decimal.Decimal)):
                raise ValueError(f'Property "ResampleNumber" of "PvtGenerateVelocitiesAndTimesRequest" is not a number.')

            if int(self.resample_number) != self.resample_number:
                raise ValueError(f'Property "ResampleNumber" of "PvtGenerateVelocitiesAndTimesRequest" is not integer value.')



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass, field
from typing import Any, Dict, Union, List, Optional
import decimal
from collections.abc import Iterable
import zaber_bson
from ..ascii.axis_type import AxisType
from ..ascii.io_port_label import IoPortLabel


TestVariant = Union[AxisType, IoPortLabel, int, Optional[float], List[str]]
"""
Test case for variant support. Not part of the public API. Do not use.
"""


@dataclass
class TestVariantWireFormat:
    """
    Serialization wrapper for TestVariant variant. Not part of the public API; do not use.
    """

    variantValueType: str = ""

    axisTypeValue: AxisType = next(first for first in AxisType)

    ioPortLabelValue: IoPortLabel = field(default_factory=IoPortLabel.zero_values)

    int32Value: int = 0

    stringArrayValue: List[str] = field(default_factory=list)

    optionalDoubleValue: Optional[float] = None

    def __init__(self, value: TestVariant) -> None:
        if value is None:
            self.optionalDoubleValue = value
            self.variantValueType = 'optionalDouble'
        elif isinstance(value, Iterable):
            self.stringArrayValue = value
            self.variantValueType = 'stringArray'
        elif isinstance(value, int):
            self.int32Value = value
            self.variantValueType = 'int32'
        elif isinstance(value, (float, decimal.Decimal)):
            self.optionalDoubleValue = value
            self.variantValueType = 'optionalDouble'
        elif isinstance(value, AxisType):
            self.axisTypeValue = value
            self.variantValueType = 'AxisType'
        elif isinstance(value, IoPortLabel):
            self.ioPortLabelValue = value
            self.variantValueType = 'IoPortLabel'
        else:
            raise TypeError(f"Cannot initialize TestVariant with value of type {type(value)}")

    def convert_back(self) -> TestVariant:
        if self.variantValueType == 'AxisType':
            return self.axisTypeValue
        if self.variantValueType == 'IoPortLabel':
            return self.ioPortLabelValue
        if self.variantValueType == 'int32':
            return self.int32Value
        if self.variantValueType == 'optionalDouble':
            return self.optionalDoubleValue
        if self.variantValueType == 'stringArray':
            return self.stringArrayValue

        raise ValueError(f"Invalid variant type tag value: {self.variantValueType}")

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'TestVariantWireFormat':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return TestVariantWireFormat.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        d: Dict[str, Any] = {
            'variantValueType': self.variantValueType,
        }

        if self.variantValueType == 'AxisType':
            d['axisTypeValue'] = self.axisTypeValue.value
        elif self.variantValueType == 'IoPortLabel':
            d['ioPortLabelValue'] = self.ioPortLabelValue.to_dict()
        elif self.variantValueType == 'int32':
            d['int32Value'] = int(self.int32Value)
        elif self.variantValueType == 'optionalDouble':
            d['optionalDoubleValue'] = float(self.optionalDoubleValue) if self.optionalDoubleValue is not None else None
        elif self.variantValueType == 'stringArray':
            d['stringArrayValue'] = [str(item or '') for item in self.stringArrayValue] if self.stringArrayValue is not None else []
        else:
            raise ValueError(f"Invalid variant type tag {self.variantValueType} value")

        return d

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'TestVariantWireFormat':
        tag = data.get('variantValueType')
        if tag == 'AxisType':
            return TestVariantWireFormat(AxisType(data.get('axisTypeValue')))  # type: ignore
        if tag == 'IoPortLabel':
            return TestVariantWireFormat(IoPortLabel.from_dict(data.get('ioPortLabelValue')))  # type: ignore
        if tag == 'int32':
            return TestVariantWireFormat(data.get('int32Value'))  # type: ignore
        if tag == 'optionalDouble':
            return TestVariantWireFormat(data.get('optionalDoubleValue'))  # type: ignore
        if tag == 'stringArray':
            return TestVariantWireFormat(data.get('stringArrayValue'))  # type: ignore

        raise ValueError(f"Invalid variant type tag {tag} in response data")

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.variantValueType == 'AxisType':
            if self.axisTypeValue is None:
                raise ValueError(f'Property "AxisType" of "TestVariant" is None.')

            if not isinstance(self.axisTypeValue, AxisType):
                raise ValueError(f'Property "AxisType" of "TestVariant" is not an instance of "AxisType".')

        elif self.variantValueType == 'IoPortLabel':
            if self.ioPortLabelValue is None:
                raise ValueError(f'Property "IoPortLabel" of "TestVariant" is None.')

            if not isinstance(self.ioPortLabelValue, IoPortLabel):
                raise ValueError(f'Property "IoPortLabel" of "TestVariant" is not an instance of "IoPortLabel".')

            self.ioPortLabelValue.validate()

        elif self.variantValueType == 'int32':
            if self.int32Value is None:
                raise ValueError(f'Property "int32" of "TestVariant" is None.')

            if not isinstance(self.int32Value, (int, float, decimal.Decimal)):
                raise ValueError(f'Property "int32" of "TestVariant" is not a number.')

            if int(self.int32Value) != self.int32Value:
                raise ValueError(f'Property "int32" of "TestVariant" is not integer value.')

        elif self.variantValueType == 'optionalDouble':
            if self.optionalDoubleValue is not None:
                if not isinstance(self.optionalDoubleValue, (int, float, decimal.Decimal)):
                    raise ValueError(f'Property "optionalDouble" of "TestVariant" is not a number.')

        elif self.variantValueType == 'stringArray':
            if self.stringArrayValue is not None:
                if not isinstance(self.stringArrayValue, Iterable):
                    raise ValueError('Property "stringArray" of "TestVariant" is not iterable.')

                for i, stringArrayValue_item in enumerate(self.stringArrayValue):
                    if stringArrayValue_item is not None:
                        if not isinstance(stringArrayValue_item, str):
                            raise ValueError(f'Item {i} in property "stringArray" of "TestVariant" is not a string.')



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass, field
from typing import Any, Dict, List
import decimal
from collections.abc import Iterable
import zaber_bson
from ..unit_conversion_descriptor import UnitConversionDescriptor
from ...units import Units, UnitsAndLiterals, units_from_literals


@dataclass
class UnitConvertNativeRequest:

    value: float = 0

    values: List[float] = field(default_factory=list)

    unit: UnitsAndLiterals = Units.NATIVE

    conversion: UnitConversionDescriptor = field(default_factory=UnitConversionDescriptor.zero_values)

    @staticmethod
    def zero_values() -> 'UnitConvertNativeRequest':
        return UnitConvertNativeRequest(
            value=0,
            values=[],
            unit=Units.NATIVE,
            conversion=UnitConversionDescriptor.zero_values(),
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'UnitConvertNativeRequest':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return UnitConvertNativeRequest.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'value': float(self.value),
            'values': [float(item) for item in self.values] if self.values is not None else [],
            'unit': units_from_literals(self.unit).value,
            'conversion': self.conversion.to_dict(),
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'UnitConvertNativeRequest':
        return UnitConvertNativeRequest(
            value=data.get('value'),  # type: ignore
            values=data.get('values'),  # type: ignore
            unit=Units(data.get('unit')),  # type: ignore
            conversion=UnitConversionDescriptor.from_dict(data.get('conversion')),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.value is None:
            raise ValueError(f'Property "Value" of "UnitConvertNativeRequest" is None.')

        if not isinstance(self.value, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Value" of "UnitConvertNativeRequest" is not a number.')

        if self.values is not None:
            if not isinstance(self.values, Iterable):
                raise ValueError('Property "Values" of "UnitConvertNativeRequest" is not iterable.')

            for i, values_item in enumerate(self.values):
                if values_item is None:
                    raise ValueError(f'Item {i} in property "Values" of "UnitConvertNativeRequest" is None.')

                if not isinstance(values_item, (int, float, decimal.Decimal)):
                    raise ValueError(f'Item {i} in property "Values" of "UnitConvertNativeRequest" is not a number.')

        if self.unit is None:
            raise ValueError(f'Property "Unit" of "UnitConvertNativeRequest" is None.')

        if not isinstance(self.unit, (Units, str)):
            raise ValueError(f'Property "Unit" of "UnitConvertNativeRequest" is not Units.')

        if self.conversion is None:
            raise ValueError(f'Property "Conversion" of "UnitConvertNativeRequest" is None.')

        if not isinstance(self.conversion, UnitConversionDescriptor):
            raise ValueError(f'Property "Conversion" of "UnitConvertNativeRequest" is not an instance of "UnitConversionDescriptor".')

        self.conversion.validate()



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

from typing import Optional
from .call import call, call_async, call_sync
from .dto import requests as dto
from .dto.log_output_mode import LogOutputMode
from .dto.device_db_source_type import DeviceDbSourceType
from .dto.device_db_source import DeviceDbSource


class Library:
    """
    Access class to general library information and configuration.
    """

    @staticmethod
    def set_log_output(
            mode: LogOutputMode,
            file_path: Optional[str] = None
    ) -> None:
        """
        Sets library logging output.

        Args:
            mode: Logging output mode.
            file_path: Path of the file to open.
        """
        request = dto.SetLogOutputRequest(
            mode=mode,
            file_path=file_path,
        )
        call_sync("logging/set_output", request)

    @staticmethod
    def set_device_db_source(
            source_type: DeviceDbSourceType,
            url_or_file_path: Optional[str] = None
    ) -> None:
        """
        Sets source of Device DB data. Allows selection of a web service or a local file.

        Args:
            source_type: Source type.
            url_or_file_path: URL of the web service or path to the local file.
                Leave empty for the default URL of Zaber web service.
        """
        request = dto.SetDeviceDbSourceRequest(
            source_type=source_type,
            url_or_file_path=url_or_file_path,
        )
        call_sync("device_db/set_source", request)

    @staticmethod
    def set_device_db_sources(
            *sources: DeviceDbSource
    ) -> None:
        """
        Sets a sequence of sources. When the library needs device information,
        it will try each source in the order they are provided.

        Args:
            sources: The list of sources the library will access data from.
        """
        request = dto.SetDeviceDbLayeredSourcesRequest(
            sources=list(sources),
        )
        call_sync("device_db/set_sources", request)

    @staticmethod
    def enable_device_db_store(
            store_location: Optional[str] = None
    ) -> None:
        """
        Enables Device DB store.
        The store uses filesystem to save information obtained from the Device DB.
        The stored data are later used instead of the Device DB.

        Args:
            store_location: Specifies relative or absolute path of the folder used by the store.
                If left empty defaults to a folder in user home directory.
                Must be accessible by the process.
        """
        request = dto.ToggleDeviceDbStoreRequest(
            toggle_on=True,
            store_location=store_location,
        )
        call_sync("device_db/toggle_store", request)

    @staticmethod
    def disable_device_db_store() -> None:
        """
        Disables Device DB store.
        """
        request = dto.ToggleDeviceDbStoreRequest(
        )
        call_sync("device_db/toggle_store", request)

    @staticmethod
    def is_device_db_store_enabled() -> bool:
        """
        Checks if the Device DB store is currently enabled.

        Returns:
            True if the Device DB store is enabled.
        """
        request = dto.EmptyRequest(
        )
        response = call_sync(
            "device_db/check_store",
            request,
            dto.BoolResponse.from_binary)
        return response.value

    @staticmethod
    def clear_device_db_store() -> None:
        """
        Clears the Device DB store on the local filesystem.
        Note: If the device DB was enabled with a custom store location, store files will be removed in that location.
        """
        request = dto.EmptyRequest(
        )
        call("device_db/clear_store", request)

    @staticmethod
    async def clear_device_db_store_async() -> None:
        """
        Clears the Device DB store on the local filesystem.
        Note: If the device DB was enabled with a custom store location, store files will be removed in that location.
        """
        request = dto.EmptyRequest(
        )
        await call_async("device_db/clear_store", request)

    @staticmethod
    def set_internal_mode(
            mode: bool
    ) -> None:
        """
        Disables certain customer checks (like FF flag).

        Args:
            mode: Whether to turn internal mode on or off.
        """
        request = dto.SetInternalModeRequest(
            mode=mode,
        )
        call_sync("library/set_internal_mode", request)

    @staticmethod
    def set_idle_polling_period(
            period: int
    ) -> None:
        """
        Sets the period between polling for IDLE during movements.
        Caution: Setting the period too low may cause performance issues.

        Args:
            period: Period in milliseconds.
                Negative value restores the default period.
        """
        request = dto.IntRequest(
            value=period,
        )
        call_sync("library/set_idle_polling_period", request)

    @staticmethod
    def check_version() -> None:
        """
        Deprecated: Calling this function is no longer necessary as the check happens automatically.

        Throws an error if the version of the loaded shared library does not match the caller's version.
        """
        request = dto.CheckVersionRequest(
            host="py",
            version="8.0.4",
        )
        call_sync("library/check_version", request)


Library.check_version()
